@@ -7,7 +7,7 @@
 **
 **
 ** This file may be distributed and/or modified under the terms of the
-** GNU General Public License version 2 as published by the Free Software 
+** GNU General Public License version 2 as published by the Free Software
 ** Foundation and appearing in the file gpl-2.0.txt included in the
 ** packaging of this file.
 **
@@ -15,12 +15,12 @@
 ** but WITHOUT ANY WARRANTY; without even the implied warranty of
 ** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 ** GNU General Public License for more details.
-** 
+**
 ** You should have received a copy of the GNU General Public License
 ** along with this program; if not, write to the Free Software
 ** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 **
-** This copyright notice MUST APPEAR in all copies of the script!  
+** This copyright notice MUST APPEAR in all copies of the script!
 **
 **********************************************************************/
 
@@ -33,7 +33,8 @@
 #include "rs_pattern.h"
 #include "rs_patternlist.h"
 
-#include <QPolygon>
+#include <QPainterPath>
+
 
 /**
  * Constructor.
@@ -53,8 +54,8 @@ RS_Hatch::RS_Hatch(RS_EntityContainer* parent,
  * Validates the hatch.
  */
 bool RS_Hatch::validate() {
-	bool ret = true;
-	
+    bool ret = true;
+
     // loops:
     for (RS_Entity* l=firstEntity(RS2::ResolveNone);
             l!=NULL;
@@ -67,17 +68,17 @@ bool RS_Hatch::validate() {
         }
     }
 
-	return ret;
+    return ret;
 }
 
 
-    
+
 RS_Entity* RS_Hatch::clone() {
     RS_Hatch* t = new RS_Hatch(*this);
     t->setOwner(isOwner());
     t->initId();
     t->detach();
-	t->hatch = NULL;
+    t->hatch = NULL;
     return t;
 }
 
@@ -105,8 +106,8 @@ void RS_Hatch::calculateBorders() {
 
     RS_EntityContainer::calculateBorders();
 
-	RS_DEBUG->print("RS_Hatch::calculateBorders: size: %f,%f", 
-		getSize().x, getSize().y);
+    RS_DEBUG->print("RS_Hatch::calculateBorders: size: %f,%f",
+        getSize().x, getSize().y);
 
     activateContour(false);
 }
@@ -114,12 +115,12 @@ void RS_Hatch::calculateBorders() {
 
 
 /**
- * Updates the Hatch. Called when the 
+ * Updates the Hatch. Called when the
  * hatch or it's data, position, alignment, .. changes.
  */
 void RS_Hatch::update() {
-	RS_DEBUG->print("RS_Hatch::update");
-	RS_DEBUG->print("RS_Hatch::update: contour has %d loops", count());
+    RS_DEBUG->print("RS_Hatch::update");
+    RS_DEBUG->print("RS_Hatch::update: contour has %d loops", count());
 
     if (updateRunning) {
         return;
@@ -147,12 +148,12 @@ void RS_Hatch::update() {
         return;
     }
 
-	if (!validate()) {
-		RS_DEBUG->print(RS_Debug::D_WARNING,
-			"RS_Hatch::update: invalid contour in hatch found");
+    if (!validate()) {
+        RS_DEBUG->print(RS_Debug::D_WARNING,
+            "RS_Hatch::update: invalid contour in hatch found");
         updateRunning = false;
-		return;
-	}
+        return;
+    }
 
     // search pattern:
     RS_DEBUG->print("RS_Hatch::update: requesting pattern");
@@ -461,31 +462,31 @@ void RS_Hatch::update() {
  * Activates of deactivates the hatch boundary.
  */
 void RS_Hatch::activateContour(bool on) {
-	RS_DEBUG->print("RS_Hatch::activateContour: %d", (int)on);
+    RS_DEBUG->print("RS_Hatch::activateContour: %d", (int)on);
     for (RS_Entity* e=firstEntity(); e!=NULL;
             e=nextEntity()) {
         if (!e->isUndone()) {
             if (!e->getFlag(RS2::FlagTemp)) {
-				RS_DEBUG->print("RS_Hatch::activateContour: set visible");
+                RS_DEBUG->print("RS_Hatch::activateContour: set visible");
                 e->setVisible(on);
             }
-			else {
-				RS_DEBUG->print("RS_Hatch::activateContour: entity temp");
-			}
+            else {
+                RS_DEBUG->print("RS_Hatch::activateContour: entity temp");
+            }
+        }
+        else {
+            RS_DEBUG->print("RS_Hatch::activateContour: entity undone");
         }
-		else {
-			RS_DEBUG->print("RS_Hatch::activateContour: entity undone");
-		}
     }
-	RS_DEBUG->print("RS_Hatch::activateContour: OK");
+    RS_DEBUG->print("RS_Hatch::activateContour: OK");
 }
 
 
 /**
  * Overrides drawing of subentities. This is only ever called for solid fills.
  */
-void RS_Hatch::draw(RS_Painter* painter, RS_GraphicView* view, 
-	double /*patternOffset*/) {
+void RS_Hatch::draw(RS_Painter* painter, RS_GraphicView* view,
+    double /*patternOffset*/) {
 
     if (!data.solid) {
         for (RS_Entity* se=firstEntity();
@@ -497,13 +498,9 @@ void RS_Hatch::draw(RS_Painter* painter, RS_GraphicView* view,
         return;
     }
 
+    QList<QPolygon> paClosed;
     QPolygon pa;
-    QPolygon jp;   // jump points
-    uint s=0;
-    uint sj=0;
-    int lastX=0;
-    int lastY=0;
-    bool lastValid=false;
+//    QPolygon jp;   // jump points
 
     // loops:
     if (needOptimization==true) {
@@ -522,8 +519,8 @@ void RS_Hatch::draw(RS_Painter* painter, RS_GraphicView* view,
 
     // loops:
     for (RS_Entity* l=firstEntity(RS2::ResolveNone);
-            l!=NULL;
-            l=nextEntity(RS2::ResolveNone)) {
+         l!=NULL;
+         l=nextEntity(RS2::ResolveNone)) {
 
         l->setLayer(getLayer());
 
@@ -532,135 +529,92 @@ void RS_Hatch::draw(RS_Painter* painter, RS_GraphicView* view,
 
             // edges:
             for (RS_Entity* e=loop->firstEntity(RS2::ResolveNone);
-                    e!=NULL;
-                    e=loop->nextEntity(RS2::ResolveNone)) {
+                 e!=NULL;
+                 e=loop->nextEntity(RS2::ResolveNone)) {
 
                 e->setLayer(getLayer());
                 switch (e->rtti()) {
                 case RS2::EntityLine: {
-                        RS_Line* line = (RS_Line*)e;
+                    RS_Line* line=static_cast<RS_Line*>(e);
+                    QPoint pt1(RS_Math::round(view->toGuiX(line->getStartpoint().x)),
+                               RS_Math::round(view->toGuiY(line->getStartpoint().y)));
+                    QPoint pt2(RS_Math::round(view->toGuiX(line->getEndpoint().x)),
+                               RS_Math::round(view->toGuiY(line->getEndpoint().y)));
 
-                        int x1 = RS_Math::round(
-                                     view->toGuiX(line->getStartpoint().x));
-                        int y1 = RS_Math::round(
-                                     view->toGuiY(line->getStartpoint().y));
-                        int x2 = RS_Math::round(
-                                     view->toGuiX(line->getEndpoint().x));
-                        int y2 = RS_Math::round(
-                                     view->toGuiY(line->getEndpoint().y));
+//                    if (! (pa.size()>0 && (pa.last() - pt1).manhattanLength()<=2)) {
+//                        jp<<pt1;
+//                    }
 
-                        if (lastValid && (lastX!=x1 || lastY!=y1)) {
-                            jp.resize(++sj);
-                            jp.setPoint(sj-1, x1, y1);
-                        }
-
-                        pa.resize(++s);
-                        pa.setPoint(s-1, x1, y1);
-
-                        pa.resize(++s);
-                        pa.setPoint(s-1, x2, y2);
-
-                        lastX = x2;
-                        lastY = y2;
-                        lastValid=true;
-                    }
+                    pa<<pt1<<pt2;
+                }
                     break;
 
                 case RS2::EntityArc: {
-                        RS_Arc* arc = (RS_Arc*)e;
+//                    QPoint pt1(RS_Math::round(view->toGuiX(e->getStartpoint().x)),
+//                               RS_Math::round(view->toGuiY(e->getStartpoint().y)));
+//                    if (! (pa.size()>0 && (pa.last() - pt1).manhattanLength()<=2)) {
+//                        jp<<pt1;
+//                    }
 
-                        int x1 = RS_Math::round(
-                                     view->toGuiX(arc->getStartpoint().x));
-                        int y1 = RS_Math::round(
-                                     view->toGuiY(arc->getStartpoint().y));
-                        int x2 = RS_Math::round(
-                                     view->toGuiX(arc->getEndpoint().x));
-                        int y2 = RS_Math::round(
-                                     view->toGuiY(arc->getEndpoint().y));
+                    QPolygon pa2;
+                    RS_Arc* arc=static_cast<RS_Arc*>(e);
+                    painter->createArc(pa2, view->toGui(arc->getCenter()),
+                                       view->toGuiDX(arc->getRadius()),
+                                       arc->getAngle1(),
+                                       arc->getAngle2(),
+                                       arc->isReversed());
+                    pa<<pa2;
 
-                        if (lastValid && (lastX!=x1 || lastY!=y1)) {
-                            jp.resize(++sj);
-                            jp.setPoint(sj-1, x1, y1);
-                        }
-
-                        pa.resize(++s);
-                        pa.setPoint(s-1, x1, y1);
-
-                        QPolygon pa2;
-                        painter->createArc(pa2, view->toGui(arc->getCenter()),
-                                           view->toGuiDX(arc->getRadius()),
-                                           arc->getAngle1(),
-                                           arc->getAngle2(),
-                                           arc->isReversed());
-
-                        pa.resize(s+pa2.size());
-                        pa.putPoints(s, pa2.size(), pa2);
-                        s+=pa2.size()-1;
-
-                        pa.resize(++s);
-                        pa.setPoint(s-1, x2, y2);
-
-                        lastX = x2;
-                        lastY = y2;
-                        lastValid=true;
-                    }
+                }
                     break;
 
                 case RS2::EntityCircle: {
-                        RS_Circle* circle = (RS_Circle*)e;
+                    RS_Circle* circle = static_cast<RS_Circle*>(e);
+                    QPoint pt1(RS_Math::round(view->toGuiX(circle->getCenter().x+circle->getRadius())),
+                               RS_Math::round(view->toGuiY(circle->getCenter().y)));
+//                    if (! (pa.size()>0 && (pa.last() - pt1).manhattanLength()<=2)) {
+//                        jp<<pt1;
+//                    }
 
-                        int x1 = RS_Math::round(
-                                     view->toGuiX(circle->getCenter().x
-                                                  + circle->getRadius()));
-                        int y1 = RS_Math::round(
-                                     view->toGuiY(circle->getCenter().y));
-                        int x2 = x1;
-                        int y2 = y1;
-
-                        if (lastValid && (lastX!=x1 || lastY!=y1)) {
-                            jp.resize(++sj);
-                            jp.setPoint(sj-1, x1, y1);
-                        }
-
-                        pa.resize(++s);
-                        pa.setPoint(s-1, x1, y1);
-
-                        QPolygon pa2;
-                        painter->createArc(pa2, view->toGui(circle->getCenter()),
-                                           view->toGuiDX(circle->getRadius()),
-                                           0.0,
-                                           2*M_PI,
-                                           false);
-
-                        pa.resize(s+pa2.size());
-                        pa.putPoints(s, pa2.size(), pa2);
-                        s+=pa2.size()-1;
-
-                        pa.resize(++s);
-                        pa.setPoint(s-1, x2, y2);
-
-                        lastX = x2;
-                        lastY = y2;
-                        lastValid=true;
-                    }
+                    QPolygon pa2;
+                    painter->createArc(pa2, view->toGui(circle->getCenter()),
+                                       view->toGuiDX(circle->getRadius()),
+                                       0.0,
+                                       2*M_PI,
+                                       false);
+                    pa<<pa2;
+                }
                     break;
 
                 default:
                     break;
                 }
+                if( pa.size()>2 && pa.first() == pa.last()) {
+                    paClosed<<pa;
+                    pa.clear();
+                }
+
             }
 
         }
     }
-
-    for (int i=(int)jp.count()-1; i>=0; --i) {
-        pa.resize(++s);
-        pa.setPoint(s-1, jp.point(i));
+    if(pa.size()>2){
+        pa<<pa.first();
+        paClosed<<pa;
     }
+       QPainterPath path;
+    for(int i=0;i<paClosed.size();i++){
+        path.addPolygon(paClosed.at(i));
+    }
+        painter->setBrush(painter->getPen().getColor());
+        painter->disablePen();
+        painter->drawPath(path);
 
-    painter->setBrush(painter->getPen().getColor());
-    painter->disablePen();
-    painter->drawPolygon(pa);
+//    pa<<jp;
+
+//    painter->setBrush(painter->getPen().getColor());
+//    painter->disablePen();
+//    painter->drawPolygon(pa);
 
 }
 
